<?php

include_once("inc/getwebobject.class.php");

class Article
{
  var $db   = null;
  var $id   = null;
  var $path = null;
  
  var $bandwith_used = null;
  
  var $err_long;
  var $err_short;
  var $C_SUCCESSFUL      = 0;
  var $C_NO_ARTICLE      = 1;
  var $C_NO_CONTENT      = 2;
  var $C_NO_PATH_SET     = 3;
  var $C_ARTICLE_EXISTS  = 10;
  var $C_ARTICLE_BLOCKED = 11;
  var $C_DATABASE_ERROR  = 20;
  var $C_EXTERNAL_ERROR  = 99;
  
  
  function Article($key)
  {
    if($this->__isPath($key))
    { $this->path = $key;}
    $this->id = basename($key);
    
    $this->bandwith_used = false;
  } 
  
  function show()
  {
    global $tb_prefix;
    
    if(!$this->isSaved())
    {
      $errno = $this->save($article);
      if($errno)
      {
        return($errno);
      }
    }
    
		$this->db->query("SELECT content
		                  FROM   ${tb_prefix}news
		                  WHERE  id='$this->id'");
		$rs = $this->db->fetch_array();
		
		if(empty($rs['content']))
		{
		  return($this->C_DATABASE_ERROR);
		}
		
		echo $rs['content'];
		return($this->C_SUCCESSFUL);
  }
  
  function save()
  {
    global $conf, $tb_prefix;
    
    if(empty($this->path))
    {
      return($this->C_NO_PATH_SET);
    }
    
    // is the article already saved?
    if($this->isSaved())
    { return($this->C_ARTICLE_EXISTS);}
    
    $this->__getArticleCategories($this->path, $maincat, $subcat);
    
    // is the article's category blocked?
    if($conf->isCategoryBlocked($maincat) || $conf->isCategoryBlocked($subcat))
    { return($this->C_ARTICLE_BLOCKED);}
    
    // construct the new path to the printable article version
    $path_printable = preg_replace("/(\w*),(\w*),(\w*),(\w*)\.html/", "$1,$2,druck-$3,$4.html", $this->path);
    
    /*
      fetch the remote file
    */
    $this->bandwith_used = true;
    $file = new GetWebObject("www.spiegel.de", 80, $path_printable);
    $errno = $file->fetch();

    if($errno)
    { 
      $this->err_long  = $file->getLongErrDesc($errno);
      $this->err_short = $file->getShortErrDesc($errno);
      return($this->C_EXTERNAL_ERROR);
    }

    $content = $file->getContent();
    
    if(!$this->__isContentAvailable($content))
    {
      return($this->C_NO_ARTICLE);
    }

    /*
      get the article's title from the content
    */
    preg_match("/<title>Druckversion \- (.*) \- .+ \- .+<\/title>/is", $content, $match);
    $title = addslashes($match[1]);
    
    /*
      get the article's timestamp from the content
    */
    $str = substr($content, strpos($content, "<BODY"), strpos($content, "URL:"));
    preg_match("/SPIEGEL ONLINE \- ([\w\.,;&: ]*)/i", $str, $match);
    
    if(empty($match[1]))
    { $str = date("d F Y H:i");}
    else
    { $str = $match[1];}

    $search = array("/&nbsp;/",
                    "/\./",
                    "/,/",
                    "/Januar/",
                    "/Februar/",
                    "/M&auml;rz/",
                    "/April/",
                    "/Mai/",
                    "/Juni/",
                    "/Juli/",
                    "/Oktober/",
                    "/Dezember/");

    $replace = array(" ",
                     "",
                     "",
                     "January",
                     "February",
                     "March",
                     "April",
                     "May",
                     "June",
                     "July",
                     "October",
                     "December");
    $str = preg_replace($search, $replace, $str);
    $timestamp = strtotime($str);
    
    /*
      remove some unused/unwanted stuff from the content
    */
    $search  = array("/^.*<body .*?>/is",                                 // remove HTML header
                     "/<\/body>.*/is",                                    // remove HTML footer
                     "/<!\-\- IVW.*<!\-\- \/IVW \-\->/is",                // remove IVW spyware
                     "/<hr.*?Alle Rechte vorbehalten.*?<hr .*?><br>/s",   // "suppress" the -erm- copyright notice
                     "/<hr .*>\n<br><br>/s",                              // remove the hr at the bottom (links)
                     "/(<p>([\ \n])*){2,}/is");                           // remove multiple linefeeds via <p>
    $replace = array("",
                     "",
                     "",
                     "",
                     "",
                     "<p>");
    $content = preg_replace($search, $replace, $content);
    
    // if some regexps are too greedy...
    if(empty($content))
    {
      return($this->C_NO_CONTENT);
    }

    /*
      save the images
    */    
    $this->__saveImages($content);
    
    /*
      manipulate the hyperlinks in the document
    */
    $search  = array("/<img src=\"\/static.*?\".*?>/is",
                     "/src=\"\/img\//",
                     "/href=\"\//");
    $replace = array("",
                     "src=\"image.php?q=",
                     "href=\"?p=article&q=/");
    $content = preg_replace($search, $replace, $content);
    
    /*
      save the article
    */
    if(!empty($content))
    {
      $content = addslashes($content);
      $this->db->query("INSERT INTO ${tb_prefix}news
                                    (id, title, maincat, subcat, content, cdate)
                        VALUES      ('$this->id', '$title', '$maincat', '$subcat',
                                     '$content', $timestamp )
                       ");
      return($this->C_SUCCESSFUL);
    }
    return($this->C_NO_CONTENT);
  }
  
  function getTitle()
  {
    global $tb_prefix;
    
    if(empty($this->id) || empty($this->db))
    { return("");}
    
    $id = addslashes($this->id);
    $this->db->query("SELECT title
                      FROM   ${tb_prefix}news
                      WHERE  id = '${id}'");
    if($this->db->affected_rows() == 0)
    { return("Article does not exist");}
    else
    {
      $rs = $this->db->fetch_array();
      return($rs['title']);
    }
  }
  
  function getColoredTitle($color1="black", $color2="red")
  {
    $title = $this->getTitle();
    $parts = explode(":", $title);
    if(count($parts) >= 2)
    {
      if(empty($color1))
      { $first = array_shift($parts).":";}
      else
      { $first = "<font color=\"${color1}\">".array_shift($parts).":</font>";}
      
      if(empty($color2))
      { $last = implode(":", $parts);}
      else
      { $last  = "<font color=\"${color2}\">".implode(":", $parts)."</font>";}
      $title = $first." ".$last;
    }
    else
    {
      $title = $parts[0];
    }
    return($title);
  }
  
  function getTimestamp()
  {
    global $tb_prefix;
    
    if(empty($this->id) || empty($this->db))
    { return(0);}
    
    $id = addslashes($this->id);
    $this->db->query("SELECT cdate
                      FROM   ${tb_prefix}news
                      WHERE  id = '${id}'");
    if($this->db->affected_rows() == 0)
    { return("Article does not exist");}
    else
    {
      $rs = $this->db->fetch_array();
      return($rs['cdate']);
    }
  }
  
  function getHeader()
  {
    global $tb_prefix;
    
    if(empty($this->id) || empty($this->db))
    { return(0);}
    
    $id = addslashes($this->id);
    $this->db->query("SELECT content
                      FROM   ${tb_prefix}news
                      WHERE  id = '${id}'");
    if($this->db->affected_rows() == 0)
    { return("Article does not exist");}
    else
    {
      $rs = $this->db->fetch_array();
      $content = $rs['content'];

      preg_match("/<font size=\"\+3\">.*?<b>(.*?)<\/b>/is", $content, $match);
      if(isset($match[1]))
      { return($match[1]);}
      return("");
    }
  }
  
  function isBandwithUsed()
  {
    return($this->bandwith_used);
  }
  
  function isSaved()
  {
    global $tb_prefix;
    
    if(!is_object($this->db))
    {
      echo "Article::isSaved() failed. No database object present<br>";
      return(false);
    }

    $this->db->query("SELECT 1
                      FROM   ${tb_prefix}news 
                      WHERE  id='".$this->id."'");

    if($this->db->affected_rows() == 0)
    { return(false);}
    else
    { return(true);}
  }
  
  function __isContentAvailable(&$content)
  {
    if(preg_match("/premium-link.net|Automatische Weiterleitung|server-fehler/i", $content))
    { return(false);}
    return(true);
  }
  
  function __isPath($string)
  {
    if(preg_match("/^\//", $string))
    { return(1);}
    return(0);
  }
  
  function __getArticleCategories($path, &$maincat, &$subcat)
  {
    /*
      $path can be something like
      "/maincat/subcat/xxx.html" or "/maincat/xxx.html"
    */
    
    // cut off the first slash
    $path = substr($path, 1, strlen($path));
    $parts = explode("/", $path);
    
    if(isset($parts[1]))
    { 
      $maincat = $parts[0];
      if($parts[1] == $this->id)
      { $subcat = "";}
      else
      { $subcat = $parts[1];}
    }
  }
  
  function __saveImages(&$content)
  {
    global $conf, $tb_prefix;
    
    // get all image urls
    preg_match_all("/<img src=\"\/img\/(.*?)\" .*?>/is",$content, $images);

    $pic = new GetWebObject("www.spiegel.de", 80, "");
    if(is_array($images))
    {
      foreach($images[1] as $name)
      {
        $pic->setPath("/img/".$name);

        $this->db->query("SELECT 1
                          FROM   ${tb_prefix}images
                          WHERE  filename='$name'");

        // only store if non-existant in db
        if($this->db->affected_rows() == 0)
        {
          $pic->fetch();
          $binary_junk = addslashes($pic->getContent());
          $header      = $pic->getHeader();
          $type        = $header['Content-Type'];

					if(strlen($pic->getContent()) <= $conf->getMaxImageSize() )
					{
		        $this->db->query("INSERT INTO ${tb_prefix}images
		                                      (binary_junk, filename, filetype)
		                          VALUES      ('$binary_junk', '$name', '$type')");
		      }
        }
      }
    }
  }
  
  function setDbHandle(&$db)
  {
    if(is_object($db))
    { $this->db = $db;}
  }
  
  function getShortErrDesc($no)
  {
    switch($no)
    {
      case $this->C_SUCCESSFUL:
        return("successful");
      case $this->C_NO_ARTICLE:
        return("not available");
      case $this->C_NO_CONTENT:
        return("no content");
      case $this->C_NO_PATH_SET:
        return("no url path set");
      case $this->C_ARTICLE_EXISTS:
        return("already exists");
      case $this->C_ARTICLE_BLOCKED:
        return("blocked");
      case $this->C_DATABASE_ERROR:
        return("database error");
      case $this->C_EXTERNAL_ERROR:
        return($this->err_short);
      default:
        return("unknown error ($no)");
    }
  }

  function getLongErrDesc($no)
  {
    switch($no)
    {
      case $this->C_SUCCESSFUL:
        return("Operation successful");
      case $this->C_NO_ARTICLE:
        return("Article is not available");
      case $this->C_NO_CONTENT:
        return("Article content is gone");
      case $this->C_NO_PATH_SET:
        return("No URL path available to save");
      case $this->C_ARTICLE_EXISTS:
        return("Article already exists");
      case $this->C_ARTICLE_BLOCKED:
        return("Article is blocked by configuration");
      case $this->C_DATABASE_ERROR:
        return("A database error occured");
      case $this->C_EXTERNAL_ERROR:
        return($this->err_long);
      default:
        return("Unknown error code ($no)");
    }
  }
}




?>